package com.carey.lucene;

import java.io.File;
import java.io.FileReader;
import java.io.Reader;
import java.util.Date;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.Field.Store;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.store.FSDirectory;

/**
 * @author Carey Zhou
 * 
 *         lucene demo.
 * */
public class LuceneDemo {
	private final static String SEARCHCONTENT = "search_content";
	private final static String QUERYRESULT = "query_result";

	public static void main(String[] args) {
		String strIndexDir = "E:\\Personal-Files";
		String strDataDir = "E:\\Personal-Files\\Android\\AndroidFramework\\base\\core\\java\\android\\app";

		try {
			fileIndexer(strIndexDir, strDataDir);
		} catch (Exception e) {
			e.printStackTrace();
		}

		String queryStr = "Application";

		try {
			fileSearcher(queryStr, strIndexDir);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * create index with Lucene for java files, indexDir is the directory that
	 * hosts Lucene's index files, dataDir is the directory that hosts the text
	 * files that to be indexed.
	 */
	public static void fileIndexer(String strIndexDir, String strDataDir)
			throws Exception {
		File indexDir = new File(strIndexDir);
		File dataDir = new File(strDataDir);

		Analyzer luceneAnalyzer = new StandardAnalyzer();
		File[] dataFiles = dataDir.listFiles();
		IndexWriter indexWriter = new IndexWriter(indexDir, luceneAnalyzer,
				true);
		long startTime = new Date().getTime();

		for (int i = 0; i < dataFiles.length; i++) {
			if (dataFiles[i].isFile()
					&& dataFiles[i].getName().endsWith(".java")) {
				System.out.println("Indexing file "
						+ dataFiles[i].getCanonicalPath());
				Document document = new Document();
				Reader txtReader = new FileReader(dataFiles[i]);
				document.add(new Field(QUERYRESULT, dataFiles[i]
						.getCanonicalPath().getBytes(), Store.COMPRESS));
				document.add(new Field(SEARCHCONTENT, txtReader));
				indexWriter.addDocument(document);
			}
		}
		indexWriter.optimize();
		indexWriter.close();
		long endTime = new Date().getTime();

		System.out.println("It takes " + (endTime - startTime)
				+ " milliseconds to create index for the files in directory "
				+ dataDir.getPath());
	}

	/**
	 * searching on an existing Lucene index, indexDir is the directory that
	 * hosts the Lucene index
	 */
	public static void fileSearcher(String queryStr, String strIndexDir)
			throws Exception {
		File indexDir = new File(strIndexDir);

		FSDirectory directory = FSDirectory.getDirectory(indexDir, false);
		IndexSearcher searcher = new IndexSearcher(directory);
		if (!indexDir.exists()) {
			System.out.println("The Lucene index is not exist");
			return;
		}

		Term term = new Term(SEARCHCONTENT, queryStr.toLowerCase());
		TermQuery luceneQuery = new TermQuery(term);
		Hits hits = searcher.search(luceneQuery);
		for (int i = 0; i < hits.length(); i++) {
			Document document = hits.doc(i);
			System.out.println("File: "
					+ new String(document.getField(QUERYRESULT).binaryValue()));
		}
	}

}
